package com.daqing.financial.hrms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.daqing.financial.hrms.dao.DeptDao;
import com.daqing.financial.hrms.dao.EmployeeDao;
import com.daqing.financial.hrms.service.DeptService;
import com.daqing.framework.domain.hrms.DepartmentVO;
import com.daqing.framework.domain.hrms.DeptEntity;
import com.daqing.framework.domain.hrms.EmployeeEntity;
import com.daqing.framework.domain.hrms.ext.EmployeeVO;
import com.daqing.framework.domain.hrms.ext.UserDeptVO;
import com.daqing.framework.domain.hrms.request.DeptAddRequest;
import com.daqing.framework.domain.hrms.response.HrmsCode;
import com.daqing.framework.exception.ExceptionCast;
import com.daqing.framework.utils.SnowflakeIdUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;


@Service("deptService")
public class DeptServiceImpl extends ServiceImpl<DeptDao, DeptEntity> implements DeptService {

    @Autowired
    private DeptDao deptDao;

    @Autowired
    private EmployeeDao employeeDao;

    /**
     * 保存一条部门信息
     *
     * @param deptAddRequest 新增部门请求体对象
     * @return 保存成功或失败
     */
    @Override
    public boolean saveDept(DeptAddRequest deptAddRequest) {
        //1、校验部门名称是否合法（为空与是否已存在）
        String deptName = deptAddRequest.getName();
        if (StringUtils.isEmpty(deptName)) {
            //抛出部门名称非法异常
            ExceptionCast.cast(HrmsCode.DEPT_NAME_ILLEGAL);
        }
        //判断上级部门id是否为空
        if (deptAddRequest.getParentId() == null) {
            //如果上级部门id为空，赋值为0
            deptAddRequest.setParentId(0L);
        }
        //得到上级部门id
        Long parentId = deptAddRequest.getParentId();
        //根据name和上级部门id查询数据条数，如果条数大于0，说明数据库中已经存在此部门名称
        int countByName = this.count(new QueryWrapper<DeptEntity>()
                .eq("name", deptName)
                .eq("parent_id", parentId));
        if (countByName > 0) {
            //抛出部门名称非法异常
            ExceptionCast.cast(HrmsCode.DEPT_NAME_ILLEGAL);
        }
        //实体类
        DeptEntity deptEntity = new DeptEntity();
        //设置id
        //deptEntity.setId(SnowflakeIdUtils.getRandomid());
        //属性拷贝，将上级部门id（如果有）、部门名称设置到实体类
        BeanUtils.copyProperties(deptAddRequest, deptEntity);
        //2、判断部门上级id是否为空，为空则此部门是一级部门，否则查询并计算部门层级，并判断部门层级是否大于5级或小于1级
        if (parentId == 0L) {
            //说明是添加一级部门
            //设置层级为1
            deptEntity.setLevel(1);
        } else {
            //添加非一级部门
            //根据上级部门id查询上级部门信息
            DeptEntity byId = this.getById(parentId);
            if (byId == null) {
                //抛出部门id不存在异常
                ExceptionCast.cast(HrmsCode.DEPT_ID_NOT_EXSIT);
            }
            //根据上级部门层级计算当前部门层级
            int level = byId.getLevel() + 1;
            if (level > 5 || level < 1) {
                //部门层级大于5级或小于1级，抛出异常
                ExceptionCast.cast(HrmsCode.DEPT_LEVEL_ILLEGAL);
            }
            deptEntity.setLevel(level);
        }
        //设置排序
        //查询当前数据库当前parent_id下最大排序号
        Integer maxSort = this.baseMapper.getMaxSort(parentId);
        if (maxSort != null) {
            deptEntity.setSort(maxSort + 1);
        }
        //设置创建时间和修改时间
        deptEntity.setCreateTime(new Date());
        deptEntity.setMotifyTime(new Date());
        //3、保存部门信息到数据库
        return this.save(deptEntity);
    }

    /**
     * 树形结构部门查询
     *
     * @return
     */
    @Override
    public List<DeptEntity> listWithTree() {
        return getDeptTreeList(this.list(), 0L);
    }

    private List<DeptEntity> getDeptTreeList(List<DeptEntity> list, Long parentId) {
        return list.stream()
                .filter(dept -> parentId.equals(dept.getParentId()))
                .peek(dept -> {
                    dept.setChildren(getDeptTreeList(list, dept.getId()));

                })
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort())))
                .collect(Collectors.toList());
    }

    @Override
    public boolean updateDept(DeptEntity dept) {
        String deptName = dept.getName();

        if (StringUtils.isEmpty(deptName)) {
            ExceptionCast.cast(HrmsCode.DEPT_NAME_ILLEGAL);
        }

        int count = this.count(new QueryWrapper<DeptEntity>()
                .eq("name", deptName)
                .notIn("id", dept.getId()));
        if (count > 0) {
            ExceptionCast.cast(HrmsCode.DEPT_NAME_ILLEGAL);
        }

        return this.updateById(dept);
    }

    @Override
    public boolean deleteByIds(Long[] ids) {
        //判断该部门下没有员工，才可删除
//        List<Long> collect = Arrays.stream(ids)
//                .filter(id -> this.getBaseMapper().getEmpDeptCount(id) > 0)
//                .collect(Collectors.toList());
        List<Long> list = Arrays.asList(ids);
        list.forEach(id -> {
            if (this.getBaseMapper().getEmpDeptCount(id) > 0) {
                ExceptionCast.cast(HrmsCode.DEPT_EMP_EXSIT);
            }
        });
        return this.removeByIds(list);
    }

    /**
     * 获取所有的部门及部门下的员工信息
     *
     * @return
     */
    @Override
    public List<DepartmentVO> listDepartmentAndEmployeeTree(String account) {
        return this.getListDepartmentAndEmployeeTree(deptDao.listDepartment(), 0L, account);
    }


    private List<DepartmentVO> getListDepartmentAndEmployeeTree(List<DepartmentVO> list, Long parentId, String account) {
        return list.stream()
                .filter(dept -> parentId.equals(dept.getParentId())) // 过滤出部门id为所选id的部门信息(也为递归的结果条件，找不到所对应的部门id)
                .peek(dept -> {
                    dept.setChildren(getListDepartmentAndEmployeeTree(list, dept.getId(), account)); // 设置该部门的子部门:递归调用

                    List<UserDeptVO> empList = employeeDao.employeeListByDeptId(dept.getId(),account);
                    System.out.println("empList========================" + empList);

                    dept.setUserDept(employeeDao.employeeListByDeptId(dept.getId(),account));

                    //dept.setEmployee(employeeDao.listEmployeeByDeptId(dept.getId())); // 设置该部门下面的所有员工
                })
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort()))) // 根据所选排序信息进行排序
                .collect(Collectors.toList());
    }

    /**
     * 根据条件获取所有部门及部门下的员工信息
     */
/*    @Override
    public List<DepartmentVO> listDeptAndEmpTree(String name, List<Long> empIds) {

        return this.getListDepartmentAndEmployeeTree(deptDao.listDepartment(), 0L, name, empIds);
    }

    private List<DepartmentVO> getListDepartmentAndEmployeeTree(List<DepartmentVO> list, Long parentId, String name, List<Long> empIds) {
        return list.stream()
                .filter(dept -> parentId.equals(dept.getParentId())) // 过滤出部门id为所选id的部门信息(也为递归的结果条件，找不到所对应的部门id)
                .peek(dept -> {
                    dept.setChildren(getListDepartmentAndEmployeeTree(list, dept.getId())); // 设置该部门的子部门:递归调用
                    dept.setEmployee(employeeDao.listEmployeeByDeptIdAndEmpId(dept.getId(), name, empIds)); // 设置该部门下面的所有员工
                })
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort()))) // 根据所选排序信息进行排序
                .collect(Collectors.toList());
    }*/

    /**
     * 获取该部门下子部门及部门下的员工
     */
    @Override
    public DepartmentVO getDepartmentAndEmployee(Long id) {
        DepartmentVO departmentVO = new DepartmentVO();
        if (id == null) {
            List<DepartmentVO> departments = deptDao.getDepartmentByParentId(0L);
            departmentVO.setChildren(departments);
            return departmentVO;
        } else {
            List<DepartmentVO> departments = deptDao.getDepartmentByParentId(id);
            List<EmployeeVO> employeeS = employeeDao.listEmployeeByDeptId(id);
            departmentVO.setChildren(departments);
            departmentVO.setEmployee(employeeS);
            return departmentVO;
        }
    }

    /**
     * 根据员工id获取对应部门下所有的客户经理
     *
     * @param id 员工id
     * @return
     */
    @Override
    public List<EmployeeVO> listEmployeeById(Long id) {

        return this.getBaseMapper().listEmployeeById(id);
    }

    /**
     * 根据部门id获取该部门所有的员工id和姓名
     */
    public List<EmployeeVO> listEmployeeByDeptId(Long deptId) {

        return this.getBaseMapper().listEmployeeByDeptId(deptId);
    }

    /**
     * 获取所有的部门及部门下的员工信息
     *
     * @return
     */
    public List<DepartmentVO> listEmployeeTrees(String account) {
        return this.getListEmployeeTrees(deptDao.listDepartment(), 0L, account);
    }

    /**
     * 获取所有的部门id和名称(用于团队效率部门筛选)
     */
    @Override
    public List<Map<Long, String>> listDepartment() {
        return this.getBaseMapper().listDepartments();
    }


    private List<DepartmentVO> getListEmployeeTrees(List<DepartmentVO> list, Long parentId, String account) {
        return list.stream()
                .filter(dept -> parentId.equals(dept.getParentId())) // 过滤出部门id为所选id的部门信息(也为递归的结果条件，找不到所对应的部门id)
                .peek(dept -> {
                    dept.setChildren(getListEmployeeTrees(list, dept.getId(), account)); // 设置该部门的子部门:递归调用

                    List<UserDeptVO> empList = employeeDao.getListEmployeeTrees(dept.getId(),account);

                    dept.setUserDept(employeeDao.getListEmployeeTrees(dept.getId(),account));
                })
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort()))) // 根据所选排序信息进行排序
                .collect(Collectors.toList());
    }
}